-- GLOBAL IGNORE LIST

----------------------------------
-- Global Ignore List Variables --
----------------------------------

local _, L          = ...
local safeToLoad    = false
local addonLoaded   = false
local serverName    = GetCVar("realmName")
local faction       = nil
local maxIgnoreSize = 50
local firstClear    = false
local firstPrune    = false
local pruneDays     = 0
local ignoreAdded   = false
local indentUI      = 0
local timer         = 0
local loadedTime    = GetTime()
local nameUI        = ""
 
local BlizzardAddIgnore        = nil
local BlizzardDelIgnore        = nil
local BlizzardAddOrDelIgnore   = nil
local BlizzardIgnoreListUpdate = nil

-------------
-- UNICODE --
-------------

local function GetWord (str, wordnumber)

	local len     = strlen(str)
	local count   = 1
	local res     = ""
	local words   = 0
	local nextInc = true
	local c
	
	while count <= len do
		if nextInc then
			words = words + 1
			
			if words > wordnumber then
				break
			end
		end
		
		c = strbyte(str, count)
		
		nextInc = (c == 32) and nextInc == false
		
		if words == wordnumber and c ~= 32 then
			res = res .. string.char(c)
		end
		
		count = count + 1
	end
	
	return res
end

local function prettyServer (name)

	if name == nil then return nil end
	
	local len  = strlen(name)
	local sb   = strbyte
	local char = string.char
	local res  = ""
	local c1
	local c
	local gotP = false
	
	for count = 1, len do
		c  = sb(name, count)
		c1 = sb(name, count+1) or 32
		
		if ((c1 > 64 and c1 < 91) or (c1 == 40)) and gotP == false then
			res = res .. char(c) .. " "
			
			if c1 == 40 then gotP = true end
		else
			res = res .. char(c)
		end
	end
	
	return res
end

local function Proper (name, okSpaces)

	if name == nil then return nil end

	local len    = strlen(name)
	local count  = 1
	local res    = ""
	local needUp = true
	local gotOP  = false
	local c
	local char   = string.char
	local upper  = string.upper
	local lower  = string.lower
	local sb     = strbyte
	
	while count <= len do
		c = sb(name, count)
				
		if c < 32 or c > 126 then
			res    = res .. char(c)
			needUp = false
		else
			if c ~= 32 or okSpaces == true then
				if needUp then
					res = res .. upper(char(c))
				else
					res = res .. lower(char(c))
				end
			
				gotOP  = (c == 40 or gotOP) and (c ~= 41)
				needUp = (c == 32 or c == 45 or gotOP == true)
			end
		end
		
		count  = count + 1
	end
	
	return res
end

local function strDown (str)

	local len    = strlen(str)
	local count  = 1
	local res    = ""
	local c
	
	while count <= len do
		c = strbyte(str, count)
		
		if c < 32 or c > 126 then
			res = res .. string.char(c)
		else
			res = res .. string.lower(string.char(c))
		end
		
		count = count + 1
	end
	
	return res
end

----------------------------------
-- Global Ignore List Functions --
----------------------------------

local function ShowMsg (msg)

	print ("|cff33ff99Global Ignore: |cffffffff" .. msg)

end

local function OnOff (value)

	if value == nil or value == false then
		return "|cffffff00" .. L["OFF"] .. "|cffffffff"
	elseif value == true then
		return "|cffffff00" .. L["ON"] .. "|cffffffff"
	else
		return "|cffff0000nil"
	end
end

local function dayString (value)

	if value == 1 then
		return L["DAY"]
	else
		return L["DAYS"]
	end
end

local function trim (str)

	local n = str:find"%S"
	return n and str:match(".*%S", n) or ""
 
end

local function dateToJulianDate (dateStr)
	
	if dateStr == nil then
		return 0
	end

	local monthList = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" }
	local words     = {}

	for word in dateStr:gmatch("%w+") do
		table.insert(words, word)
	end
	
	local day   = tonumber(words[1])
	local year  = tonumber(words[3])
	local month = 0

	for key, value in pairs(monthList) do
	
		if value == words[2] then
			month = tonumber(key)
			
			break
		end
	end

	--print ("monthStr="..monthStr.." month="..month.." day="..day.." year="..year)

	if (not month) or (not day) or (not year) or (month < 1) or (month > 12) or (day < 1) or (day > 31) or (year < 2014) then
		return 0
	end
	
	local calc1 = (month - 14) / 12
	local calc2 = day - 32075 + (1461 * (year + 4800 + calc1) / 4)
	
	calc2 = calc2 + (367 * (month - 2 - calc1 * 12) / 12)
	calc2 = calc2 - (3 * ((year + 4900 + calc1) / 100) / 4)
	
	return calc2
end

local function daysFromToday (dateStr)

	local addDate = dateToJulianDate(dateStr)
	local today   = dateToJulianDate(date("%d %b %Y"))
	
	if addDate == 0 then
		return -1
	else
	
		if addDate < today then
			return math.floor(today - addDate)
		else
			return math.floor(addDate - today)
		end
	end
end

local function AddToList(newname, newfaction, newnote, newtype)

	local index = #GlobalIgnoreDB.ignoreList+1
	
	GlobalIgnoreDB.ignoreList[index] = newname
	GlobalIgnoreDB.factionList[index] = newfaction
	GlobalIgnoreDB.dateList[index] = date("%d %b %Y")
	GlobalIgnoreDB.notes[index] = (newnote or "")
	GlobalIgnoreDB.expList[index] = GlobalIgnoreDB.defexpire
	GlobalIgnoreDB.typeList[index] = (newtype or "player")
	
end

local function RemoveFromList(index)

	if index <= #GlobalIgnoreDB.ignoreList then
		table.remove(GlobalIgnoreDB.ignoreList, index)
		table.remove(GlobalIgnoreDB.factionList, index)
		table.remove(GlobalIgnoreDB.dateList, index)
		table.remove(GlobalIgnoreDB.notes, index)
		table.remove(GlobalIgnoreDB.expList, index)
		table.remove(GlobalIgnoreDB.typeList, index)
	end
end

local function addServer (name)

	if not name then return nil end

	if string.find(name, "-", 1, true) == nil then
		return name .. "-" .. serverName
	end
	
	return name
end

local function getServer (name, def)

	local index = string.find(name, "-", 1, true)
	
	if index ~= nil then
		return string.sub(name, index + 1, string.len(name))
	end
	
	if def then return def else return serverName end
end

local function removeServer (name, strict)

	result = name
	
	local index = string.find(name, "-", 1, true)
	
	if strict == nil then
		strict = false
	end
	
	if index ~= nil then
		local server = Proper(string.sub(name, index + 1, string.len(name)));
		
		if server == Proper(serverName) or strict == true then
			result = string.sub(name, 1, index - 1)
		end
	end
	
	return result	
end

local function isServerMatch (server1, server2)

	return Proper(server1) == Proper(server2)
	
end

local function hasIgnored (name)

	local result = 0
	
	name = Proper(addServer(name))
		
	for count = 1, GetNumIgnores() do
	
		if name == Proper(addServer(GetIgnoreName(count))) then
			result = count
			
			break
		end
	end
	
	return result
end

local function hasNPCIgnored (name)

	if not name then return 0 end
	
	for count = 1, #GlobalIgnoreDB.ignoreList do
	
		if GlobalIgnoreDB.ignoreList[count] == name and GlobalIgnoreDB.typeList[count] == "npc" then
			return count
		end
	end
	
	return 0
end

local function hasServerIgnored (name)

	if not name then return 0 end
	
	for count = 1, #GlobalIgnoreDB.ignoreList do
	
		if GlobalIgnoreDB.ignoreList[count] == name and GlobalIgnoreDB.typeList[count] == "server" then
			return count
		end
	end
	
	return 0
end

local function hasGlobalIgnored (name)

	if not name then return 0 end

	for count = 1, #GlobalIgnoreDB.ignoreList do
	
		if GlobalIgnoreDB.ignoreList[count] == name and GlobalIgnoreDB.typeList[count] == "player" then
			return count
		end
	end
	
	return 0
end

local function hasAnyIgnored (name)

	if not name then return 0 end

	for count = 1, #GlobalIgnoreDB.ignoreList do
	
		if GlobalIgnoreDB.ignoreList[count] == name then
			return count
		end
	end
	
	return 0
end

local function ShowIgnoreList (param)

	local days   = tonumber(param)
	local sName  = ""
	
	if not days then
	
		days  = 0
		sName = param
		
		if not sName then
			sName = ""
		end
	end
	
	if days > 0 then
		ShowMsg("|cffffff00" .. format(L["LIST_1"], days))		
	else
		if sName ~= "" then
			sName = Proper(sName)
			
			if sName == "Npc" then
				ShowMsg("|cffffff00".. L["LIST_2"])
			else			
				if sName == "Server" then
					sName = serverName		
				end
				
				ShowMsg("|cffffff00" .. format(L["LIST_3"], sName))
			end
		else
			ShowMsg("|cffffff00" .. L["LIST_4"])
		end
	end
	
	local count = 0
	
	for key,value in pairs(GlobalIgnoreDB.ignoreList) do
	
		local ok   = true
		local type = "P"
		
		if GlobalIgnoreDB.typeList[key] == "npc" then
			type = "N"
		elseif GlobalIgnoreDB.typeList[key] == "server" then
			type = "S"
		end
	
		if days > 0 then
		
			ok = daysFromToday(GlobalIgnoreDB.dateList[key]) >= days
			
		elseif sName ~= "" then
		
			ok = (type == "N" and sName == "Npc") or (type == "P" and isServerMatch(sName, getServer(value))) or (type == "S" and isServerMatch(sName, value))

		end
		
		if ok then
			local str = "  (" .. key .. ") [" .. type.. "] " .. value .. " (" .. (GlobalIgnoreDB.factionList[key] or "Unknown") .. ") " .. "[".. daysFromToday(GlobalIgnoreDB.dateList[key]) .. " "..L["DAYS"] .. "]"
			
			if GlobalIgnoreDB.notes[key] ~= "" then
				str = str .." (" .. GlobalIgnoreDB.notes[key] .. ")"
			end
			
			ShowMsg(str)

			count = count + 1			
		end
		
	end
	
	ShowMsg("|cffffff00" .. format(L["LIST_5"], count))
end

local function ResetIgnoreDB()

	GlobalIgnoreDB = {
		chatmsg		= true,
		sameserver	= true,
		samefaction	= true,
		chatfilter	= true,
		openWithFriends	= true,
		attachFriends   = true,
		defexpire	= 0,
		ignoreList	= {},
		factionList	= {},
		dateList	= {},
		notes		= {},
		expList		= {},
		typeList	= {}
	}
	
	GlobalIgnoreImported = false
end

local function SyncIgnoreList(silent)

	if silent == nil then
		silent = false
	end
	
	for count = 1, GetNumIgnores() do
	
		local name = addServer(GetIgnoreName(count))
	
		if hasGlobalIgnored(name) == 0 then
			name = removeServer(name)
			
			if not silent then
				ShowMsg (format(L["SYNC_1"], name))
			end			
			
			BlizzardDelIgnore (hasIgnored(name))
		end
	end
	
	local count = 0
	
	while count < #GlobalIgnoreDB.dateList do
		count = count + 1
		
		if GlobalIgnoreDB.expList[count] > 0 and daysFromToday(GlobalIgnoreDB.dateList[count]) >= GlobalIgnoreDB.expList[count] then
			local name = addServer(GlobalIgnoreDB.ignoreList[count])		
			DelIgnore(name)
			count = 0
		end
	end
	
	for key,value in pairs(GlobalIgnoreDB.ignoreList) do
	
		if GlobalIgnoreDB.typeList[key] == "player" then
		
			local name = addServer(value)
			
			if hasIgnored(name) == 0 then
		
				local ok = (GlobalIgnoreDB.factionList[key] == faction)				

				if ok then		
					ok = (isServerMatch(serverName, getServer(name))) or (GlobalIgnoreDB.sameserver == false)				
				end
			
				if ok then
					if GetNumIgnores() < maxIgnoreSize then
						name = removeServer(name)
			
						if not silent then
							ShowMsg (format(L["SYNC_2"], name))
						end
			
						BlizzardAddIgnore(name)
					end
				end
			end
		end
	end	
end

local function PruneIgnoreList (days, doit)

	if days == nil or days <= 0 then		
		return 0
	end

	local targets = 0
	local count   = 0
	
	while count < #GlobalIgnoreDB.dateList do
		count = count + 1
	
		if daysFromToday(GlobalIgnoreDB.dateList[count]) >= days then
			targets = targets + 1
			
			local name = addServer(GlobalIgnoreDB.ignoreList[count])
					
			if doit == true then
				DelIgnore(name)
				count = 0
			end
		end
	end
	
	return targets
end

local function ApplicationStartup(self)

	if addonLoaded or safeToLoad == false then
		return
	end
	
	ShowMsg(L["LOAD_1"])
		
	faction = UnitFactionGroup("player")
		
	if GlobalIgnoreDB == nil then
		
		ResetIgnoreDB()	
	end
	
	-- set missing defaults or upgrade if needed
		
	if not GlobalIgnoreDB.defexpire then
		GlobalIgnoreDB.defexpire = 0
	end
	
	if GlobalIgnoreDB.openWithFriends == nil then
		GlobalIgnoreDB.openWithFriends = true
	end
	
	if not GlobalIgnoreDB.attachFriends == nil then
		GlobalIgnoreDB.attachFriends = true
	end
	
	if GlobalIgnoreDB.asknote == nil then
		GlobalIgnoreDB.asknote = true
	end
		
	if not GlobalIgnoreDB.expList then
		GlobalIgnoreDB.expList = {}
			
		for  count = 1, #GlobalIgnoreDB.ignoreList do
			GlobalIgnoreDB.expList[count] = 0
		end
	end
	
	if not GlobalIgnoreDB.typeList then
		GlobalIgnoreDB.typeList = {}
		
		for count = 1, #GlobalIgnoreDB.ignoreList do
			GlobalIgnoreDB.typeList[count] = "player"
		end
	end
	
	if GlobalIgnoreDB.revision == nil then
		GlobalIgnoreDB.revision = 1
		
		for count = 1, #GlobalIgnoreDB.ignoreList do
			GlobalIgnoreDB.ignoreList[count] = Proper(GlobalIgnoreDB.ignoreList[count])
		end
	end
	
	loadedTime = GetTime()
		
	--
		
	if GlobalIgnoreImported == nil or GlobalIgnoreImported ~= true then
	
		local ignores = GetNumIgnores()	
		local added   = 0
			
		if (ignores > 0) and (GlobalIgnoreDB.chatmsg == nil or GlobalIgnoreDB.chatmsg == true) then
			ShowMsg(L["LOAD_2"])
		end
			
		for count = 1, ignores do
			
			local name = GetIgnoreName(count)
				
			if removeServer(name, true) ~= "Unknown" then
				
				name = Proper(addServer(name))

				if hasGlobalIgnored(name) == 0 then
					added = added + 1
						
					AddToList(name, faction)
										
					if GlobalIgnoreDB.chatmsg == nil or GlobalIgnoreDB.chatmsg == true then
						ShowMsg (format(L["LOAD_3"], name))
					end
				end
			end
				
		end
			
		GlobalIgnoreImported = true
	end
						
	-- Check for GlobalIgnoreDB nil values and set defaults here if needed
		
	if GlobalIgnoreDB.chatfilter == nil then
		GlobalIgnoreDB.chatfilter = true
	end

	SyncIgnoreList(GlobalIgnoreDB.chatmsg == nil or GlobalIgnoreDB.chatmsg == false)

	if GetNumIgnores() >= maxIgnoreSize then
		ShowMsg(format(L["LOAD_4"], maxIgnoreSize))		
	end
		
	addonLoaded = true
	
	self:UnregisterEvent("IGNORELIST_UPDATE")
end

local function EventHandler (self, event, sender, ...)

	--print ("event=".. (event or "nil"))
	
	if event == "IGNORELIST_UPDATE" then
		safeToLoad = true
	elseif event == "PLAYER_LOGIN" then
		if not safeToLoad and not addonLoaded then
		
			timer = 0
		
			self:SetScript("OnUpdate", function(Self, elapsed)
			
				timer = timer + elapsed
							
				if timer < 3 then
					return
				end
												
				self:SetScript("OnUpdate", nil)
				self:Hide()
				
				safeToLoad = true
				
				ApplicationStartup(self)
			end)
		end
	end
	
	if safeToLoad == true then
		ApplicationStartup(self)
	end
end

function SlashCmdList.GIGNORE (msg)

	msg = strDown(msg)
	
	local args   = {}
	local argStr = ""
	local count  = 1

	local str = GetWord(msg, count)
	
	while str ~= "" do
		
		table.insert(args, str)
		
		if count == 2 then
			argStr = str
		elseif count > 2 then
			argStr = argStr .. " ".. str
		end
		
		count = count + 1
		str   = GetWord(msg, count)
	end
	
	if args[1] == "clear" then
	
		if firstClear and args[2] ~= nil and args[2] == "confirm" then	
			ResetIgnoreDB()
			ShowMsg(L["CMD_2"])
			SyncIgnoreList(GlobalIgnoreDB.chatmsg == nil or GlobalIgnoreDB.chatmsg == false)
			firstClear = false
		else
			ShowMsg("|cffff0000" .. L["CMD_1"])
			firstClear = true
		end
		
	elseif args[1] == "defexpire" then
	
		if tonumber(args[2]) then
			GlobalIgnoreDB.defexpire = tonumber(args[2])
			
			ShowMsg (format(L["CMD_3"], GlobalIgnoreDB.defexpire, dayString(GlobalIgnoreDB.defexpire)))
		end
		
	elseif msg == "asknote true" or msg == "asknote on" then
	
		GlobalIgnoreDB.asknote = true
		ShowMsg (L["CMD_4"])

	elseif msg == "asknote false" or msg == "asknote off" then

		GlobalIgnoreDB.asknote = false
		ShowMsg (L["CMD_5"])
		
	elseif msg == "showmsg true" or msg == "showmsg on" then
	
		GlobalIgnoreDB.chatmsg = true
		ShowMsg (L["CMD_6"])

	elseif msg == "showmsg false" or msg == "showmsg off" then
	
		GlobalIgnoreDB.chatmsg = false	
		ShowMsg (L["CMD_7"])
	
	elseif msg == "filter true" or msg == "filter on" then

		GlobalIgnoreDB.chatfilter = true
		ShowMsg(L["CMD_8"])

	elseif msg == "filter false" or msg == "filter off" then

		GlobalIgnoreDB.chatfilter = false
		ShowMsg(L["CMD_9"])
		
	elseif msg == "sameserver true" or msg == "sameserver on" then
	
		GlobalIgnoreDB.sameserver = true
		ShowMsg(L["CMD_10"])

	elseif msg == "sameserver false" or msg == "sameserver off" then
	
		GlobalIgnoreDB.sameserver = false
		ShowMsg(L["CMD_11"])
		
	elseif args[1] == "list" then
	
		ShowIgnoreList(argStr)
		
	elseif (args[1] == "add" or args[1] == "ignore") then
	
		argStr = (trim(Proper(argStr)) or "")
		
		if argStr == "" then
			argStr, server =  UnitName("target")
					
			if server ~= nil then
				argStr = argStr .. "-"..server
			end
			
			if argStr == nil or not UnitPlayerControlled("target") then
				argStr = ""
			end
		end
		
		if argStr ~= "" then
			AddIgnore (Proper(argStr))
		end
		
	elseif (args[1] == "remove" or args[1] == "delete") and args[2] ~= nil and args[2] ~= "" then
	
		if tonumber(argStr) then
		
			local str = GlobalIgnoreDB.typeList[tonumber(argStr)]
			
			if str == "npc" then
				ShowMsg (format(L["CMD_12"], Proper(GlobalIgnoreDB.ignoreList[tonumber(argStr)], true)))
				RemoveFromList(tonumber(argStr))
			else
				DelIgnore(args[2], true)
			end
		else
			argStr = Proper(argStr, true)
			
			local npcIndex = hasNPCIgnored(argStr)
		
			if npcIndex > 0 then
				ShowMsg (format(L["CMD_12"], argStr))
				RemoveFromList(npcIndex)
			else	
				DelIgnore (args[2], true)
			end
		end
		
	elseif (args[1] == "server" or args[1] == "addserver") and args[2] ~= nil and args[2] ~= "" then

		AddOrDelServer(argStr)

	elseif (args[1] == "npc" or args[1] == "addnpc") then
	
		AddOrDelNPC(argStr)
		
	elseif args[1] == "expire" and args[2] ~= nil and args[2] ~= "" and tonumber(args[3]) then

		if tonumber(args[2]) then
			local index = tonumber(args[2])

			if (index > 0) and (index <= #GlobalIgnoreDB.ignoreList) then
			
				GlobalIgnoreDB.expList[index] = tonumber(args[3])
				ShowMsg(format(L["CMD_14"], GlobalIgnoreDB.ignoreList[index], tonumber(args[3])))
			end

		else
			local name        = Proper(addServer(args[2]))
			local playerIndex = hasGlobalIgnored(name)

			if playerIndex > 0 then
				GlobalIgnoreDB.expList[playerIndex] = tonumber(args[3])
				ShowMsg(format(L["CMD_14"], name, tonumber(args[3])))
			end
		end
		
	elseif args[1] == "gui" then	
	
		GUI()
		
	elseif args[1] == "sync" then
	
		SyncIgnoreList()
		
	elseif args[1] == "prune" then
	
		if args[2] == "confirm" and firstPrune == true then
		
			PruneIgnoreList(pruneDays, true)
			
			firstPrune = false
		elseif args[2] == nil or tonumber(args[2]) == nil then
		
			ShowMsg(L["CMD_15"])
		else
			if firstPrune == false then

				pruneDays = tonumber(args[2])
				
				ShowMsg(format(L["CMD_16"], pruneDays))
				ShowMsg(format(L["CMD_17"], PruneIgnoreList(pruneDays, false)))
				
				firstPrune = true
			end
		end

	else
		ShowMsg (L["HELP_1"])
		ShowMsg ("")
		ShowMsg ("  " .. L["HELP_2"])
		ShowMsg ("  " .. L["HELP_3"])
		ShowMsg ("  " .. L["HELP_4"])
		ShowMsg ("  " .. L["HELP_5"])
		ShowMsg ("  " .. L["HELP_6"])
		ShowMsg ("  " .. L["HELP_7"])
		ShowMsg ("  " .. L["HELP_8"])
		ShowMsg ("  " .. L["HELP_15"])
		ShowMsg ("  " .. L["HELP_16"])		
		ShowMsg ("")
		ShowMsg ("  " .. format(L["HELP_9"], OnOff(GlobalIgnoreDB.chatfilter)))
		ShowMsg ("  " .. format(L["HELP_10"], OnOff(GlobalIgnoreDB.chatmsg)))
		ShowMsg ("  " .. format(L["HELP_11"], OnOff(GlobalIgnoreDB.sameserver)))
		ShowMsg ("  " .. format(L["HELP_12"], GlobalIgnoreDB.defexpire))
		ShowMsg ("  " .. format(L["HELP_13"], OnOff(GlobalIgnoreDB.asknote)))
		ShowMsg ("")
		ShowMsg (L["HELP_14"])
	end
end

-----------------------------
-- Global Ignore List Main --
-----------------------------

local frame = CreateFrame("FRAME")

frame:RegisterEvent("IGNORELIST_UPDATE")
frame:RegisterEvent("PLAYER_LOGIN")

SLASH_GIGNORE1 = "/gignore"
SLASH_GIGNORE2 = "/gi"
SLASH_GIGNORE3 = "/gil"

frame:SetScript("OnEvent", EventHandler)

serverName = Proper(serverName)

------------------------
-- Function Pre Hooks --
------------------------

BlizzardAddIgnore      = AddIgnore
BlizzardDelIgnore      = DelIgnore
BlizzardAddOrDelIgnore = AddOrDelIgnore

AddIgnore = function(name)

	if (not name or name == "") then

		name, server = UnitName("target")
					
		if server ~= nil then
			name = name .. "-"..server
		end
	end
	
	if (not name or name == "") then
		return
	end

	local note  = ""
	local space = string.find(name, " ")
	local days  = GlobalIgnoreDB.defexpire
	
	if space then
		note = string.sub(name, space + 1)
		name = string.sub(name, 0, space - 1)
		
		space = string.find(note, " ")
		
		if space then
			if tonumber(string.sub(note, 0, space - 1)) then
				days = tonumber(string.sub(note, 0, space - 1))
				note = string.sub(note, space + 1)
			end
		else
			if tonumber(note) then
				days = tonumber(note)
				note = ""
			end
		end
	end
	
	name = Proper(addServer(name))
	
	if Proper(addServer(UnitName("player"))) ~= name then

		local index = hasGlobalIgnored(name)
	
		if index == 0 then
			AddToList(name, faction, note)
			
			if GlobalIgnoreDB.asknote == true then
			
				nameUI = name				
				StaticPopup_Show("GIL_REASON", nameUI)			
			end
			
			ShowMsg(format(L["ADD_2"], name))
			
			if GetNumIgnores() < maxIgnoreSize then
				BlizzardAddIgnore(removeServer(name))
			end
		else
			if hasIgnored(name) > 0 then
				ShowMsg(format(L["ADD_1"], name))
			end
		
			BlizzardAddIgnore(removeServer(name))		
		end
		
		indentUI = 0

	else
		ShowMsg(L["ADD_3"])
	end	
end

DelIgnore = function(idxpos, isGIL)

	local name = ""
	
	if isGIL then
		if tonumber(idxpos) ~= nil then
			name = GlobalIgnoreDB.ignoreList[tonumber(idxpos)]
			
			if name == nil then
				return
			end
		else
			name = idxpos
		end
	else
		if tonumber(idxpos) ~= nil then			
			name = GetIgnoreName(idxpos)
		else
			name   = idxpos
			idxpos = hasIgnored(name)
		end
	end
	
	name = Proper(addServer(name))
	
	local index = hasGlobalIgnored(name)

	if index > 0 then
		ShowMsg(format(L["REM_1"], name))
		
		RemoveFromList(index)		
		
		name = removeServer(name)
		
		if hasIgnored(name) > 0 then
			BlizzardDelIgnore(name)
		end
	else
		BlizzardDelIgnore(idxpos)
	end
	
	indentUI = 0
end

AddOrDelIgnore = function(name)

	if (not name or name == "") then

		name, server = UnitName("target")
					
		if server ~= nil then
			name = name .. "-"..server
		end
	end
	
	if (not name or name == "") then
		return
	end
	
	name = Proper(addServer(name))
	
	local index = hasGlobalIgnored(name)

	if index == 0 then
		AddIgnore(name)
	else
		DelIgnore(index, true)
	end	
end

AddOrDelNPC = function (argStr)

	if not tonumber(argStr) then
				
		argStr = (trim(Proper(argStr, true)) or "")
	
		if argStr == "" then
			argStr = Proper(UnitName("target"), true)
				
			if argStr == nil or UnitPlayerControlled("target") then
				argStr = ""
			end
		end
			
		if argStr ~= "" then

			local npcIndex = hasNPCIgnored(argStr)
		
			if npcIndex > 0 then
				local name = GlobalIgnoreDB.ignoreList[npcIndex]
			
				ShowMsg (format(L["CMD_12"], name))
				RemoveFromList(npcIndex)
			else
				ShowMsg (format(L["CMD_13"], argStr))
				AddToList(argStr, faction, "", "npc")
			end
		end
	end
end

AddOrDelServer = function (sName)

	if not sName then return end

	if tonumber(sName) then
	
		local sIndex = tonumber(sName)
		
		if (sIndex > 0) and (sIndex <= #GlobalIgnoreDB.ignoreList) and (GlobalIgnoreDB.typeList[sIndex] == "server") then
		
			ShowMsg(format(L["CMD_19"], GlobalIgnoreDB.ignoreList[sIndex]))
			RemoveFromList(sIndex)
		end
	
	else
	
		sName = Proper(sName)
		
		local sIndex = hasServerIgnored(sName)
		
		if sIndex > 0 then
		
			ShowMsg(format(L["CMD_19"], sName))
			RemoveFromList(sIndex)
	
		else
		
			ShowMsg(format(L["CMD_18"], sName))
			AddToList(sName, faction, "", "server")
		end
	end
end


--------------------
-- Chat Filtering --
--------------------

local function chatMessageFilter (self, event, message, from, ...)

	--print ("chatMsg evt=" .. (event or "nil") .. " msg=".. (message or "nil") .. " from=" .. (from or "nil"))
	
	if addonLoaded ~= true then
		return false
	end
	
	if GlobalIgnoreDB.chatfilter == true then
	
		if event == "CHAT_MSG_MONSTER_EMOTE" or event == "CHAT_MSG_MONSTER_PARTY" or event == "CHAT_MSG_MONSTER_SAY" or
		   event == "CHAT_MSG_MONSTER_WHISPER" or event == "CHAT_MSG_MONSTER_YELL" then

	        	return (hasNPCIgnored(from) > 0)
			
		elseif event == "CHAT_MSG_SYSTEM" then
		
			if message == ERR_IGNORE_NOT_FOUND and GetTime() - loadedTime < 60 then
				return true
			end
			
			if message == ERR_IGNORE_FULL then
				return true
			end
			
			local pName = ""

			for count = 1, #GlobalIgnoreDB.ignoreList do
			
				if GlobalIgnoreDB.typeList[count] == "server" then
			
					if string.find(message, "-"..GlobalIgnoreDB.ignoreList[count], 1, true) ~= nil then
						return true
					end
				else
					-- we have to generate actual text here, otherwise, a character named "removed" or
					-- any common word could end up incorrectly filtering server messages. what a pain!

					if serverName == getServer(GlobalIgnoreDB.ignoreList[count]) then
						pName = removeServer(GlobalIgnoreDB.ignoreList[count])
					else
						pName = GlobalIgnoreDB.ignoreList[count]
					end

					local msgOffline = string.format(ERR_FRIEND_OFFLINE_S, pName)
					local msgOnline  = string.format(ERR_FRIEND_ONLINE_SS, pName, pName)
					
					if (message == msgOffline) or (message == msgOnline) then
						return true
					end
		  		end
		  	end
			
			return false

		elseif (from ~= nil) and (from ~= "") then
		
			if hasGlobalIgnored(from) > 0 or hasServerIgnored(getServer(from)) > 0 then			
			
				if event == "CHAT_MSG_WHISPER" then
					SendChatMessage(L["MSG_1"], "WHISPER", nil, from)
				end

				return true
			else
				return false
			end
		end		  	
	end
	
	return false
end

local chatEvents = (
		{
		"CHAT_MSG_ACHIEVEMENT",
		"CHAT_MSG_BATTLEGROUND",
		"CHAT_MSG_BATTLEGROUND_LEADER",
		"CHAT_MSG_CHANNEL",
		"CHAT_MSG_CHANNEL_JOIN",
		"CHAT_MSG_CHANNEL_LEAVE",
		"CHAT_MSG_EMOTE",
		"CHAT_MSG_GUILD",
		"CHAT_MSG_GUILD_ACHIEVEMENT",	
		"CHAT_MSG_MONSTER_EMOTE",
		"CHAT_MSG_MONSTER_PARTY",
		"CHAT_MSG_MONSTER_SAY",
		"CHAT_MSG_MONSTER_WHISPER",
		"CHAT_MSG_MONSTER_YELL",
		"CHAT_MSG_OFFICER",
		"CHAT_MSG_PARTY",
		"CHAT_MSG_RAID",
		"CHAT_MSG_RAID_LEADER",
		"CHAT_MSG_RAID_WARNING",
		"CHAT_MSG_SAY",
		"CHAT_MSG_SYSTEM",
		"CHAT_MSG_TEXT_EMOTE",
		"CHAT_MSG_WHISPER",
		"CHAT_MSG_WHISPER_INFORM",
		"CHAT_MSG_YELL"
		}
	)

for key, value in pairs (chatEvents) do
	ChatFrame_AddMessageEventFilter(value, chatMessageFilter)
end

------------------
-- GIL UI STUFF --
------------------

-- BLIZ HACKS
local daysUI   = {}
local expUI    = {}
local reasonUI = {}
local headerUI = {}

-- GIL UI
local WINDOW_WIDTH  = 734
local WINDOW_HEIGHT = 400
local WINDOW_OFFSET = 113
local BUTTON_HEIGHT = 20
local BUTTON_TOTAL  = math.floor((WINDOW_HEIGHT - WINDOW_OFFSET) / BUTTON_HEIGHT)
local BUTTON_WIDTH  = WINDOW_WIDTH - 44
local Tab1Frame     = nil
local IgnoreScrollFrame   = nil
local IgnoreScrollButtons = {}
local IgnoreScrollIndex   = {}
local IgnoreScrollType    = 0
local IgnoreScrollSelect  = 0
local MainFrame     = nil
local Tab2Frame     = nil
local Tab3Frame     = nil
local columnCount   = 0
local EditFrame		= nil

BlizzardIgnoreListUpdate = IgnoreList_Update

StaticPopupDialogs["GIL_REASON"] = {

  preferredIndex = STATICPOPUPS_NUMDIALOGS,
  text           = "Edit ignore note for |cffffff00%s:",
  maxLetters     = 128,
  hasEditBox	 = 1,
  whileDead      = 1,
  button1        = "Save",
  button2        = "Cancel",
  OnShow         = function(self)
			self.editBox:SetText(GlobalIgnoreDB.notes[hasAnyIgnored(nameUI)])
			self.editBox:SetFocus()
  		   end,
  OnAccept       = function(self)
			GlobalIgnoreDB.notes[hasAnyIgnored(nameUI)] = self.editBox:GetText()
			IgnoreList_Update()
                   end,
  EditBoxOnEnterPressed = function(self)
  				GlobalIgnoreDB.notes[hasAnyIgnored(nameUI)] = self:GetParent().editBox:GetText()
  				IgnoreList_Update()
  				self:GetParent():Hide()
  			end,
  EditBoxOnEscapePressed = function(self)
  				self:GetParent():Hide()
  			end
}

function setExpire (text)

	if tonumber(text) then
		GlobalIgnoreDB.expList[hasAnyIgnored(nameUI)] = tonumber(text)	
		IgnoreList_Update()
	end	
end

StaticPopupDialogs["GIL_EXPIRE"] = {

  preferredIndex = STATICPOPUPS_NUMDIALOGS,
  text           = "|cffffff00%s|cffffffff\n\n"..L["BOX_1"],
  maxLetters     = 4,
  hasEditBox	 = 1,
  whileDead      = 1,
  button1        = "Save",
  button2        = "Cancel",
  OnShow         = function(self)
  			self.editBox:SetText(GlobalIgnoreDB.expList[hasAnyIgnored(nameUI)])
  			self.editBox:SetFocus()
  		   end,
  OnAccept       = function(self)
  			setExpire(self.editBox:GetText())
                   end,
  EditBoxOnEnterPressed = function(self)
  				setExpire(self:GetParent().editBox:GetText())
  				self:GetParent():Hide()
  			end,
  EditBoxOnEscapePressed = function(self)
  				self:GetParent():Hide()
  			end
}

function IgnoreButtonEnter(self, ...)

	GameTooltip:Hide()
	GameTooltip:ClearLines()
	
	local playerIndex = hasGlobalIgnored(addServer(self.name:GetText()))
	
	if playerIndex == 0 then
		return
	end
	
	local playerNotes = GlobalIgnoreDB.notes[playerIndex]
	local daysInList  = daysFromToday(GlobalIgnoreDB.dateList[playerIndex])
	local daysExpire  = GlobalIgnoreDB.expList[playerIndex]
		
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:AddLine("|cff69CCF0" .. self.name:GetText())
	GameTooltip:AddLine(" ")
	GameTooltip:AddLine("  "..L["OVER_1"].." |cffc0c0c0" .. getServer(self.name:GetText()))
	GameTooltip:AddLine("  "..L["OVER_2"].." |cffc0c0c0" .. (GlobalIgnoreDB.factionList[playerIndex] or "Unknown"))
	GameTooltip:AddLine("  "..L["OVER_3"].." |cffc0c0c0" .. daysInList .. " " .. dayString(daysInList))
	
	if GlobalIgnoreDB.expList[playerIndex] == 0 then
		GameTooltip:AddLine("  "..L["OVER_4"].." |cffc0c0c0"..L["EXP_NVR"])
	else
		GameTooltip:AddLine("  "..L["OVER_4"].." |cffc0c0c0"..format(L["OVER_5"], daysExpire , daysExpire - daysInList))
	end
	
	GameTooltip:AddLine(" ")
	
	if playerNotes ~= nil and playerNotes ~= "" then
		GameTooltip:AddLine("|cff69CCF0"..playerNotes)
	end
	
	GameTooltip:Show()
end

function IgnoreButtonLeave(self, ...)
	GameTooltip:Hide()
end

function IgnoreButtonDoubleClick(self, ...)
	nameUI = addServer(GetIgnoreName(GetSelectedIgnore()))
	StaticPopup_Show("GIL_REASON", nameUI)
end

function IgnoreButtonClick(self, button, down)

	if button ~= "RightButton" or down == true then
		return
	end
	
	nameUI = addServer(GetIgnoreName(GetSelectedIgnore()))
		
	local IgnoreRightClickMenu = {

		{ text = GetIgnoreName(GetSelectedIgnore()), isTitle = true, notCheckable = true },
		{ text = L["RCM_1"], notCheckable = true, func = function() IgnoreButtonDoubleClick() end },
		{ text = "", disabled = true, notCheckable = true },
		{ text = L["RCM_2"], notCheckable = true, func = function() StaticPopup_Show("GIL_EXPIRE", nameUI) end },
		{ text = L["RCM_3"], notCheckable = true, func = function() GlobalIgnoreDB.expList[hasGlobalIgnored(nameUI)] = 0 IgnoreList_Update() end },
		{ text = "", disabled = true, notCheckable = true },
		{ text = L["RCM_4"], notCheckable = true, func = function() DelIgnore(GetIgnoreName(GetSelectedIgnore())) end },
		{ text = "", notCheckable = true, disabled = true },
		{ text = L["RCM_5"], notCheckable = true }
	}	
	
	local menuFrame = CreateFrame("Frame", "gil_IgnoreRightClick", UIParent, "UIDropDownMenuTemplate")

	EasyMenu(IgnoreRightClickMenu, menuFrame, "cursor", 0 , 0, "MENU")
end

---

local function GILIgnoreListUpdate (self)

	FauxScrollFrame_Update(self, #GlobalIgnoreDB.ignoreList, BUTTON_TOTAL, BUTTON_HEIGHT)
	
	if IgnoreScrollSelect > #GlobalIgnoreDB.ignoreList then
		IgnoreScrollSelect = #GlobalIgnoreDB.ignoreList
	end
	
	Tab1Frame.infotext:SetText(format(L["INFO_1"], #GlobalIgnoreDB.ignoreList))
	
	local offset  = FauxScrollFrame_GetOffset(self)
	local index   = 0
	local id      = 0
	local pName   = ""
	local pServer = ""
	local pType   = ""
	local pExpire = ""
	local temp    = 0
	
	for count = 1, BUTTON_TOTAL do
		id = count + offset
		
		if id <= #GlobalIgnoreDB.ignoreList then		
			index = IgnoreScrollIndex[id] or id			
			pName = GlobalIgnoreDB.ignoreList[index]
			temp  = string.find(pName, "-", 1, true)
			
			if temp then
				pServer = prettyServer(string.sub(pName, temp + 1, string.len(pName)))
				pName   = string.sub(pName, 1, temp - 1)
			else
				pServer = "All"
			end
			
			if GlobalIgnoreDB.typeList[index] == "player" then
				if GlobalIgnoreDB.factionList[index] == "Alliance" then
					pType = "|cff335effAlliance"
				elseif GlobalIgnoreDB.factionList[index] == "Horde" then
					pType = "|cffe60000Horde"
				else
					pType = "Unknown"
				end				  
			elseif GlobalIgnoreDB.typeList[index] == "npc" then
				pType = "|cffffff99NPC"
			else
				pType = "|cffff66ccServer"
				pServer = prettyServer(pName)
				pName = "All"				
			end	
			
			local playerExp = (GlobalIgnoreDB.expList[index] or 0)
			local daysExp   = playerExp - daysFromToday(GlobalIgnoreDB.dateList[index])

			if playerExp == 0 then
				pExpire = "|cff808080"..L["EXP_NVR"]
			elseif daysExp <= 0 then
				pExpire = "|cffff6666"..L["EXP_TDY"]
			else
				pExpire = daysExp.."d"
			end

			IgnoreScrollButtons[count].name:SetText(pName)
			IgnoreScrollButtons[count].server:SetText(pServer)
			IgnoreScrollButtons[count].type:SetText(pType)
			IgnoreScrollButtons[count].listed:SetText(daysFromToday(GlobalIgnoreDB.dateList[index]) .. "d")
			IgnoreScrollButtons[count].expire:SetText(pExpire)
			IgnoreScrollButtons[count].note:SetText("|cff69CCF0"..GlobalIgnoreDB.notes[index])
			
			IgnoreScrollButtons[count]:SetID(id)
			
			if IgnoreScrollSelect == id then
				IgnoreScrollButtons[count]:LockHighlight()
			else
				IgnoreScrollButtons[count]:UnlockHighlight()
			end
		
			IgnoreScrollButtons[count]:Show()
		else
			IgnoreScrollButtons[count]:Hide()
		end
	end
end

local function elementName(num)

	local type = GlobalIgnoreDB.typeList[num]
	
	if type == "player" or type == "npc" then
		return removeServer(GlobalIgnoreDB.ignoreList[num], true)
	elseif type == "server" then
		return "All"
	else
		return ""
	end
end

local function elementServer(num)

	local type = GlobalIgnoreDB.typeList[num]

	if type == "player" then
		return getServer(GlobalIgnoreDB.ignoreList[num])
	elseif type == "server" then
		return GlobalIgnoreDB.ignoreList[num]
	elseif type == "npc" then
		return "All"
	else
		return ""
	end
end

local function elementType(num)

	local type = GlobalIgnoreDB.typeList[num]
	
	if type == "player" then
		return GlobalIgnoreDB.factionList[num]
	else
		return type
	end
end

local function createSortedIndex(sortType)

	IgnoreScrollIndex = {}
	IgnoreScrollType  = sortType
	
	for count = 1, #GlobalIgnoreDB.ignoreList do
		IgnoreScrollIndex[count] = count
	end
	
	-- should I make one of these for each sortType for performance reasons?
	table.sort(IgnoreScrollIndex,
		function (a,b)		
			if sortType == 1 then 
				-- sort by name column only
				return GlobalIgnoreDB.ignoreList[a] < GlobalIgnoreDB.ignoreList[b]
			elseif sortType == 2 then
				return GlobalIgnoreDB.ignoreList[a] > GlobalIgnoreDB.ignoreList[b]
			elseif sortType == 3 then
				local SA = elementServer(a)
				local SB = elementServer(b)
							
				if SA < SB then
					return true
				elseif SA > SB then
					return false
				elseif SA == SB then
					return elementName(a) < elementName(b)
				end
			elseif sortType == 4 then
				local SA = elementServer(a)
				local SB = elementServer(b)
							
				if SA > SB then
					return true
				elseif SA < SB then
					return false
				elseif SA == SB then
					return elementName(a) < elementName(b)
				end
			elseif sortType == 5 then
				local SA = elementType(a)
				local SB = elementType(b)

				if SA < SB then
					return true
				elseif SA > SB then
					return false
				elseif SA == SB then
					return elementName(a) < elementName(b)
				end				
			elseif sortType == 6 then
				local SA = elementType(a)
				local SB = elementType(b)

				if SA > SB then
					return true
				elseif SA < SB then
					return false
				elseif SA == SB then
					return elementName(a) < elementName(b)
				end
			elseif sortType == 7 then
				local SA = daysFromToday(GlobalIgnoreDB.dateList[a])
				local SB = daysFromToday(GlobalIgnoreDB.dateList[b])
				
				if SA < SB then
					return true
				elseif SA > SB then
					return false
				elseif SA == SB then
					return elementName(a) < elementName(b)
				end
			elseif sortType == 8 then
				local SA = daysFromToday(GlobalIgnoreDB.dateList[a])
				local SB = daysFromToday(GlobalIgnoreDB.dateList[b])

				if SA > SB then
					return true
				elseif SA < SB then
					return false
				elseif SA == SB then
					return elementName(a) < elementName(b)
				end
				
			elseif sortType == 9 then
				local SA = GlobalIgnoreDB.expList[a]
				local SB = GlobalIgnoreDB.expList[b]

				if not SA or SA == 0 then
					SA = 100000
				else
					SA = SA - daysFromToday(GlobalIgnoreDB.dateList[a])
					if SA < 1 then SA = -1 end
				end
					
				if not SB or SB == 0 then
					SB = 100000
				else
					SB = SB - daysFromToday(GlobalIgnoreDB.dateList[b])
					if SB < 1 then SB = -1 end
				end
				
				if SA < SB then
					return true
				elseif SA > SB then
					return false
				elseif SA == SB then
					return elementName(a) < elementName(b)
				end
			elseif sortType == 10 then
				local SA = GlobalIgnoreDB.expList[a]
				local SB = GlobalIgnoreDB.expList[b]
				
				if not SA or SA == 0 then
					SA = -100000
				else
					SA = SA - daysFromToday(GlobalIgnoreDB.dateList[a])
					if SA < 1 then SA = -1 end
				end
					
				if not SB or SB == 0 then
					SB = -100000
				else
					SB = SB - daysFromToday(GlobalIgnoreDB.dateList[b])
					if SB < 1 then SB = -1 end
				end
				
				if SA  > SB then
					return true
				elseif SA < SB then
					return false
				elseif SA == SB then
					return elementName(a) < elementName(b)
				end
			elseif sortType == 11 then
				local SA = GlobalIgnoreDB.notes[a]
				local SB = GlobalIgnoreDB.notes[b]
				
				if SA == "" then SA = string.char(255) end
				if SB == "" then SB = string.char(255) end
				
				if SA < SB then
					return true
				elseif SA > SB then
					return false
				elseif SA == SB then
					return elementName(a) < elementName(b)
				end
			
			elseif sortType == 12 then
				local SA = GlobalIgnoreDB.notes[a]
				local SB = GlobalIgnoreDB.notes[b]
				
				if SA == "" then SA = string.char(255) end
				if SB == "" then SB = string.char(255) end
				
				if SA > SB then
					return true
				elseif SA < SB then
					return false
				elseif SA == SB then
					return elementName(a) < elementName(b)
				end			
			end
		end)
	
	if (IgnoreScrollFrame ~= nil) then	
		GILIgnoreListUpdate(IgnoreScrollFrame)
	end
end

IgnoreList_Update = function(...)

	BlizzardIgnoreListUpdate(...)

	if addonLoaded ~= true then
		return
	end

	if MainFrame ~= nil and Tab1Frame:IsVisible() then
		createSortedIndex(IgnoreScrollType)
	end

	local buttonWidth = 0
	
	if indentUI == 0 then
		for count = 1, IGNORES_TO_DISPLAY, 1 do
			local button = _G["FriendsFrameIgnoreButton"..count]
		
			if button then
				indentUI = max(indentUI, button.name:GetStringWidth())
			end
		end
	end

	for count = 1, IGNORES_TO_DISPLAY, 1 do
		button = _G["FriendsFrameIgnoreButton"..count]
		
		if button then

			if buttonWidth == 0 then
				buttonWidth = button:GetWidth()
			end
			
			if daysUI[count] == nil then		
				daysUI[count] = button:CreateFontString("FontString", "OVERLAY", "GameFontWhiteSmall")
		
				daysUI[count]:SetPoint("TOP", button)
				daysUI[count]:SetPoint("BOTTOMLEFT", button)
				daysUI[count]:SetJustifyH("RIGHT")
					
				button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
				
				button:HookScript("OnEnter", IgnoreButtonEnter)
				button:HookScript("OnLeave", IgnoreButtonLeave)
				button:HookScript("OnClick", IgnoreButtonClick)
				button:HookScript("OnDoubleClick", IgnoreButtonDoubleClick)
			end
			
			if expUI[count] == nil then		
				expUI[count] = button:CreateFontString("FontString", "OVERLAY", "GameFontWhiteSmall")
					
				expUI[count]:SetPoint("TOP", button)
				expUI[count]:SetPoint("BOTTOMLEFT", button)
				expUI[count]:SetJustifyH("RIGHT")
			end
			
			if reasonUI[count] == nil then
				reasonUI[count] = button:CreateFontString("FontString", "OVERLAY", "GameFontWhiteSmall")
		
				reasonUI[count]:SetPoint("TOP", button)
				reasonUI[count]:SetPoint("BOTTOMRIGHT", button)
				reasonUI[count]:SetJustifyH("LEFT")
			end				

			daysUI[count]:SetWidth(indentUI + 50)
			expUI[count]:SetWidth(indentUI + 90)
			reasonUI[count]:SetWidth(buttonWidth - indentUI - 100)
			
			if count == 1 and FauxScrollFrame_GetOffset(FriendsFrameIgnoreScrollFrame) == 0 then
			
				if headerUI[1] == nil then
					headerUI[1] = button:CreateFontString("FontString", "OVERLAY", "GameFontWhiteSmall")
					
					headerUI[1]:SetPoint("TOP", button)
					headerUI[1]:SetPoint("BOTTOMLEFT", button)
					headerUI[1]:SetPoint("LEFT", 58, 0)
					headerUI[1]:SetJustifyH("RIGHT")	
				end
				
				local ignoreNum = GetNumIgnores()
				
				if ignoreNum ~= #GlobalIgnoreDB.ignoreList then
					headerUI[1]:SetText("["..ignoreNum.."|cff00ff96+" .. (#GlobalIgnoreDB.ignoreList - ignoreNum) .."|cffffffff]")
				else
					headerUI[1]:SetText("["..ignoreNum.."]")
				end
								
				daysUI[1]:SetText("Days")
				expUI[1]:SetText("Expire")
				reasonUI[1]:SetText("Note")
			
			else
			
				if count == 1 then
					headerUI[1]:SetText("")
				end
								
				playerIndex = hasGlobalIgnored(addServer(button.name:GetText()))
				
				if playerIndex > 0 then			
					daysUI[count]:SetText(daysFromToday(GlobalIgnoreDB.dateList[playerIndex]) .. "d")
				
					local playerExp = (GlobalIgnoreDB.expList[playerIndex] or 0)
					local daysExp   = playerExp - daysFromToday(GlobalIgnoreDB.dateList[playerIndex])
				
					if playerExp == 0 then
						expUI[count]:SetText("|cff808080"..L["EXP_NVR"])
					elseif daysExp <= 0 then
						expUI[count]:SetText("|cffff6666"..L["EXP_TDY"])
					else
						expUI[count]:SetText(daysExp.."d")
					end
					
					reasonUI[count]:SetText("|cff69CCF0"..GlobalIgnoreDB.notes[playerIndex])
				end
			end				
		end
	end


end

local function GilUnitMenu (dropdownMenu, which, unit, name, userData, ...)

	if (UIDROPDOWNMENU_MENU_LEVEL > 1) then
		return
	end
		
	if (which and (which == "PLAYER" or which == "RAID_PLAYER" or which == "PARTY" or which == "TARGET")) then
	
		local target, server =  UnitName("target")
			
		if server then
			target = target .. "-"..server
		end
		
		DropDownList1.numButtons = max(0, DropDownList1.numButtons - 1)

		local info = UIDropDownMenu_CreateInfo()
		info.text = ""
		info.notCheckable = true
		info.disabled = true
		UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)	

		local info = UIDropDownMenu_CreateInfo()
				
		info.dist = 0
		info.notCheckable = 1
		
		if (which == "TARGET") then
			info.func = function() AddOrDelNPC("") end
				
			if (hasNPCIgnored(target) > 0) then
				info.text = "Remove from Global Ignore"
			else
				info.text = "Add to Global Ignore"
			end

		else
			info.func = function() AddOrDelIgnore("") end
			
			if (hasGlobalIgnored(addServer(target)) > 0) then
				info.text = "Remove from Global Ignore"
			else
				info.text = "Add to Global Ignore"
			end	
		end
		
		UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)
		
		local info = UIDropDownMenu_CreateInfo()
		info.text = "Cancel"
		info.dist = 0
		info.notCheckable = 1
		UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)
	end
end

hooksecurefunc("UnitPopup_ShowMenu", GilUnitMenu)


---------------------------
-- GLOBAL IGNORE LIST UI --
---------------------------

	local COL_NAME      = 130
	local COL_SERVER    = 150
	local COL_TYPE      = 60
	local COL_LISTED    = 53
	local COL_EXPIRE    = 56
	local COL_NOTES     = 241

local function columnClick(self)

	local id = self:GetID()

	if id == 1 then
		if IgnoreScrollType == 1 then
			createSortedIndex(2)
		else
			createSortedIndex(1)
		end
	elseif id == 2 then
		if IgnoreScrollType == 3 then
			createSortedIndex(4)
		else
			createSortedIndex(3)
		end
	elseif id == 3 then
		if IgnoreScrollType == 5 then
			createSortedIndex(6)
		else
			createSortedIndex(5)
		end
	elseif id == 4 then
		if IgnoreScrollType == 7 then
			createSortedIndex(8)
		else
			createSortedIndex(7)
		end		
	elseif id == 5 then
		if IgnoreScrollType == 9 then
			createSortedIndex(10)
		else
			createSortedIndex(9)
		end	
	elseif id == 6 then
		if IgnoreScrollType == 11 then
			createSortedIndex(12)
		else
			createSortedIndex(11)
		end	
	end
end

local function createColumn (text, width, parent)

	local p = _G[parent:GetName() .. "Header1"]
	
	if p == nil then
		columnCount = 0
	end
	
	columnCount = columnCount + 1
	
	local Header = CreateFrame("Button", parent:GetName() .. "Header" .. columnCount, parent, "WhoFrameColumnHeaderTemplate")
	Header:SetWidth(width)
	_G[parent:GetName().."Header"..columnCount.."Middle"]:SetWidth(width - 9)
	Header:SetText(text)
	Header:SetNormalFontObject("GameFontHighlight")
	Header:SetID(columnCount)
	
	if columnCount == 1 then	
		Header:SetPoint("TOPLEFT", parent, "TOPLEFT", 1, 22)
	else
		Header:SetPoint("LEFT", parent:GetName() .."Header"..columnCount - 1, "RIGHT", 0, 0)	
	end
	
	Header:SetScript("OnClick", columnClick)
end

function IgnoreScrollDoubleClick()
	nameUI = GlobalIgnoreDB.ignoreList[IgnoreScrollIndex[IgnoreScrollSelect]]
	StaticPopup_Show("GIL_REASON", nameUI)
end

function IgnoreScrollClick(self, button, down)

	if down == true then return end
	
	if IgnoreScrollSelect == self:GetID() and button == "LeftButton" then
		IgnoreScrollDoubleClick()
		return
	elseif IgnoreScrollSelect ~= self:GetID() then
		IgnoreScrollSelect = self:GetID()
		GILIgnoreListUpdate(IgnoreScrollFrame)
	end
	
	if button == "RightButton" then	
		nameUI = GlobalIgnoreDB.ignoreList[IgnoreScrollIndex[IgnoreScrollSelect]]

		local IgnoreRightClickMenu = {

			{ text = nameUI, isTitle = true, notCheckable = true },
			{ text = L["RCM_1"], notCheckable = true, func = function() IgnoreScrollDoubleClick() end },
			{ text = "", disabled = true, notCheckable = true },
			{ text = L["RCM_2"], notCheckable = true, func = function() StaticPopup_Show("GIL_EXPIRE", nameUI) end },
			{ text = L["RCM_3"], notCheckable = true, func = function() GlobalIgnoreDB.expList[IgnoreScrollIndex[IgnoreScrollSelect]] = 0 IgnoreList_Update() end },
			{ text = "", disabled = true, notCheckable = true },
			{ text = L["RCM_4"], notCheckable = true, func = function() DelIgnore(IgnoreScrollIndex[IgnoreScrollSelect], true) end },
			{ text = "", notCheckable = true, disabled = true },
			{ text = L["RCM_5"], notCheckable = true }
		}	
	
		local menuFrame = CreateFrame("Frame", "GILIgnoreRightClick", UIParent, "UIDropDownMenuTemplate")

		EasyMenu(IgnoreRightClickMenu, menuFrame, "cursor", 0 , 0, "MENU")
	end
end

local function createIgnoreButtons()

	IgnoreScrollButtons = {}
	
	for count = 1, BUTTON_TOTAL do
	
		IgnoreScrollButtons[count] = CreateFrame("Button", nil, IgnoreScrollFrame:GetParent(), "FriendsFrameIgnoreButtonTemplate")
	
		if count == 1 then
			IgnoreScrollButtons[count]:SetPoint("TOPLEFT", IgnoreScrollFrame, -1, 0)
		else
			IgnoreScrollButtons[count]:SetPoint("TOP", IgnoreScrollButtons[count - 1], "BOTTOM")
		end

		IgnoreScrollButtons[count]:SetSize(BUTTON_WIDTH, BUTTON_HEIGHT)
		IgnoreScrollButtons[count]:RegisterForClicks("LeftButtonUp", "RightButtonUp")
		IgnoreScrollButtons[count]:SetScript("OnClick", IgnoreScrollClick)		

		-- set name style
		IgnoreScrollButtons[count].name:SetWidth(COL_NAME)

		-- create server style		
		IgnoreScrollButtons[count].server = IgnoreScrollButtons[count]:CreateFontString("FontString", "OVERLAY", "GameFontHighlight")
		IgnoreScrollButtons[count].server:SetPoint("LEFT", IgnoreScrollButtons[count].name, "RIGHT", 0, 0)
		IgnoreScrollButtons[count].server:SetWidth(COL_SERVER)
		IgnoreScrollButtons[count].server:SetJustifyH("LEFT")
		
		-- create type style
		IgnoreScrollButtons[count].type = IgnoreScrollButtons[count]:CreateFontString("FontString", "OVERLAY", "GameFontHighlight")
		IgnoreScrollButtons[count].type:SetPoint("LEFT", IgnoreScrollButtons[count].server, "RIGHT", 0, 0)
		IgnoreScrollButtons[count].type:SetWidth(COL_TYPE)
		IgnoreScrollButtons[count].type:SetJustifyH("LEFT")
		
		-- create listed style
		IgnoreScrollButtons[count].listed = IgnoreScrollButtons[count]:CreateFontString("FontString", "OVERLAY", "GameFontHighlight")
		IgnoreScrollButtons[count].listed:SetPoint("LEFT", IgnoreScrollButtons[count].type, "RIGHT", -17, 0)
		IgnoreScrollButtons[count].listed:SetWidth(COL_LISTED)
		IgnoreScrollButtons[count].listed:SetJustifyH("RIGHT")
		
		-- create expire style
		IgnoreScrollButtons[count].expire = IgnoreScrollButtons[count]:CreateFontString("FontString", "OVERLAY", "GameFontHighlight")
		IgnoreScrollButtons[count].expire:SetPoint("LEFT", IgnoreScrollButtons[count].listed, "RIGHT", 3, 0)
		IgnoreScrollButtons[count].expire:SetWidth(COL_EXPIRE)
		IgnoreScrollButtons[count].expire:SetJustifyH("RIGHT")
		
		-- create note style
		IgnoreScrollButtons[count].note = IgnoreScrollButtons[count]:CreateFontString("FontString", "OVERLAY", "GameFontHighlight")
		IgnoreScrollButtons[count].note:SetPoint("LEFT", IgnoreScrollButtons[count].expire, "RIGHT", 13, 0)
		IgnoreScrollButtons[count].note:SetWidth(COL_NOTES)
		IgnoreScrollButtons[count].note:SetJustifyH("LEFT")
		IgnoreScrollButtons[count].note:SetWordWrap(false)
	end
	
	GILIgnoreListUpdate(IgnoreScrollFrame)
end

local function ButtonAdd (self, addType, index)

	if EditFrame ~= nil then
		EditFrame:Show()
		return
	end
	
	--print ("add type="..addType.. " index="..index)
end

local function CreateUIFrames()

	if MainFrame ~= nil then
		createSortedIndex(IgnoreScrollType)	
		MainFrame:Show()		
		return
	end

	----------------
	-- MAIN FRAME --
	----------------
	
	MainFrame = CreateFrame("Frame", "GIL", UIParent, "PortraitFrameTemplate")

	MainFrame:Hide()
	
	MainFrame:SetFrameStrata("DIALOG")	
	MainFrame:SetWidth(WINDOW_WIDTH)
	MainFrame:SetHeight(WINDOW_HEIGHT)
	MainFrame:SetPoint("CENTER", UIParent)
	MainFrame:SetMovable(true)
	MainFrame:EnableMouse(true)
	MainFrame:RegisterForDrag("LeftButton", "RightButton")
	MainFrame:SetClampedToScreen(true)
	
	MainFrame.title = _G["GILTitleText"]
	MainFrame.title:SetText("Global Ignore List")
	
	MainFrame:SetScript("OnMouseDown",
		function(self)
			self:StartMoving()
    			self.isMoving = true
		end)

	MainFrame:SetScript("OnMouseUp",
		function (self)
			if self.isMoving then
				self:StopMovingOrSizing()
				self.isMoving = false
			end
		end)

	local icon = MainFrame:CreateTexture("$parentIcon", "OVERLAY", nil, -8)
		
	icon:SetSize(60, 60)
	icon:SetPoint("TOPLEFT", -5, 7)
	icon:SetTexture("Interface\\FriendsFrame\\Battlenet-Portrait")
	
	local Tab1Button = CreateFrame("Button", "GILTab1", MainFrame, "CharacterFrameTabButtonTemplate")
	
	Tab1Button:SetPoint("TOPLEFT", MainFrame, "BOTTOMLEFT", 20, 1)
	Tab1Button:SetText(L["TAB_1"])
	Tab1Button:SetID("1")
	
	Tab1Button:SetScript("OnClick",
		function(self)
			--Tab3Frame:Hide()
			Tab2Frame:Hide()			
			Tab1Frame:Show()
			PanelTemplates_SetTab(MainFrame, 1)
		end)

	local Tab2Button = CreateFrame("Button", "GILTab2", MainFrame, "CharacterFrameTabButtonTemplate")
	
	Tab2Button:SetPoint("LEFT", "GILTab1", "RIGHT", -16, 0)
	Tab2Button:SetText(L["TAB_3"])
	Tab2Button:SetID("2")

	Tab2Button:SetScript("OnClick",
		function(self)
			Tab1Frame:Hide()
			Tab2Frame:Show()
			--Tab3Frame:Hide()
			PanelTemplates_SetTab(MainFrame, 2)
		end)
		
	------------------
	-- TAB 1 FRAMES --
	------------------
	
	Tab1Frame = CreateFrame("Frame", "GILFrame1", MainFrame, "InsetFrameTemplate")

	Tab1Frame:SetWidth(WINDOW_WIDTH - 19)
	Tab1Frame:SetHeight(WINDOW_HEIGHT - WINDOW_OFFSET)
	Tab1Frame:SetPoint("TOPLEFT", MainFrame, "TOPLEFT", 8, -84)
	
	createColumn(L["COL_1"], COL_NAME, Tab1Frame)
	createColumn(L["COL_2"], COL_SERVER, Tab1Frame)
	createColumn(L["COL_3"], COL_TYPE, Tab1Frame)
	createColumn(L["COL_4"], COL_LISTED, Tab1Frame)
	createColumn(L["COL_5"], COL_EXPIRE, Tab1Frame)
	createColumn(L["COL_6"], COL_NOTES, Tab1Frame)
	--createColumn("Alts", 40, Tab1Frame)

	IgnoreScrollFrame = CreateFrame("ScrollFrame", "GILIgnoreScrollFrame", Tab1Frame, "FauxScrollFrameTemplate")
	
	IgnoreScrollFrame:SetWidth  (WINDOW_WIDTH - 46)
	IgnoreScrollFrame:SetHeight (BUTTON_TOTAL * BUTTON_HEIGHT)
	IgnoreScrollFrame:SetPoint  ("TOPLEFT", 0, -4)
	
	IgnoreScrollFrame:SetScript("OnVerticalScroll",
		function(self, offset)
			FauxScrollFrame_OnVerticalScroll(self, offset, BUTTON_HEIGHT, GILIgnoreListUpdate)
		end)
		
	--
	
	Tab1Frame.infotext = Tab1Frame:CreateFontString("FontString", "OVERLAY", "GameFontHighlight")
	
	Tab1Frame.infotext:SetWidth(300)
	Tab1Frame.infotext:SetJustifyH("CENTER")
	Tab1Frame.infotext:SetPoint("TOP", 0, 46)
	
--[[	
	Tab1Frame.msgtext = Tab1Frame:CreateFontString("FontString", "OVERLAY", "GameFontHighlight")
		
	Tab1Frame.msgtext:SetWidth(300)
	Tab1Frame.msgtext:SetJustifyH("LEFT")
	Tab1Frame.msgtext:SetPoint("BOTTOMLEFT", 3, -19)
	Tab1Frame.msgtext:SetText("|cff666666Right click for menu, double click to edit note")
--]]

	createIgnoreButtons()
	
	local Button = CreateFrame("Button", "GILFrame1IgnoreButton", Tab1Frame, "UIPanelButtonTemplate")
	
	Button:SetSize(110, 22)
	Button:SetText(L["BUT_1"])
	Button:SetPoint("BOTTOMRIGHT", -1, -24)
	Button:SetScript("OnClick",
		function(self)
			if IgnoreScrollSelect > 0 then
				DelIgnore(IgnoreScrollIndex[IgnoreScrollSelect], true)
				createSortedIndex(IgnoreScrollType)
			end
		end)

--[[
	Button = CreateFrame("Button", "GILFrame1AddPlayer", Tab1Frame, "UIPanelButtonTemplate")
	Button:SetSize(110, 22)
	Button:SetText(L["BUT_2"])
	Button:SetPoint("RIGHT", "GILFrame1IgnoreButton", "LEFT", 0, 0)
	Button:SetScript("OnClick", function(self) ButtonAdd(self, 1, 0) end)

	Button = CreateFrame("Button", "GILFrame1AddNPC", Tab1Frame, "UIPanelButtonTemplate")
	Button:SetSize(110, 22)
	Button:SetText(L["BUT_3"])
	Button:SetPoint("RIGHT", "GILFrame1AddPlayer", "LEFT", 0, 0)
	Button:SetScript("OnClick", function(self) ButtonAdd(self, 2, 0) end)

	Button = CreateFrame("Button", "GILFrame1AddServer", Tab1Frame, "UIPanelButtonTemplate")
	Button:SetSize(110, 22)
	Button:SetText(L["BUT_4"])
	Button:SetPoint("RIGHT", "GILFrame1AddNPC", "LEFT", 0, 0)
	Button:SetScript("OnClick", function(self) ButtonAdd(self, 3, 0) end)

--]]
	createSortedIndex(1)

	------------------
	-- TAB 2 FRAMES --
	------------------	
	
	Tab2Frame = CreateFrame("Frame", "GILFrame2", MainFrame, "InsetFrameTemplate")
	
	Tab2Frame:Hide()

	Tab2Frame:SetWidth(WINDOW_WIDTH - 19)
	Tab2Frame:SetHeight(WINDOW_HEIGHT - WINDOW_OFFSET + 20)
	Tab2Frame:SetPoint("TOPLEFT", MainFrame, "TOPLEFT", 8, -64)
	
	Button = CreateFrame("CheckButton", "GILFrame2AskNote", Tab2Frame, "UICheckButtonTemplate")	
	Button:SetPoint("TOPLEFT", Tab2Frame, "TOPLEFT", 6, -4)
	 _G[Button:GetName().."Text"]:SetText(L["OPT_1"])
	 _G[Button:GetName().."Text"]:SetFontObject("GameFontHighlight")
	Button:SetScript("OnShow",  function(self) self:SetChecked(GlobalIgnoreDB.asknote == true) end)
	Button:SetScript("OnClick" ,function(self) GlobalIgnoreDB.asknote = (self:GetChecked() or false) end) 

	Button = CreateFrame("CheckButton", "GILFrame2OpenUI", Tab2Frame, "UICheckButtonTemplate")	
	Button:SetPoint("TOPLEFT", GILFrame2AskNote, "BOTTOMLEFT", 0, 0)
	 _G[Button:GetName().."Text"]:SetText(L["OPT_2"])
	 _G[Button:GetName().."Text"]:SetFontObject("GameFontHighlight")
	Button:SetScript("OnShow",  function(self) self:SetChecked(GlobalIgnoreDB.openWithFriends == true) end)
	Button:SetScript("OnClick" ,function(self) GlobalIgnoreDB.openWithFriends = (self:GetChecked() or false) end) 

	Button = CreateFrame("CheckButton", "GILFrame2AttachUI", Tab2Frame, "UICheckButtonTemplate")	
	Button:SetPoint("TOPLEFT", GILFrame2OpenUI, "BOTTOMLEFT", 0, 0)
	 _G[Button:GetName().."Text"]:SetText(L["OPT_3"])
	 _G[Button:GetName().."Text"]:SetFontObject("GameFontHighlight")
	Button:SetScript("OnShow",  function(self) self:SetChecked(GlobalIgnoreDB.attachFriends == true) end)
	Button:SetScript("OnClick" ,function(self) GlobalIgnoreDB.attachFriends = (self:GetChecked() or false) end) 

	Button = CreateFrame("CheckButton", "GILFrame2SameServer", Tab2Frame, "UICheckButtonTemplate")	
	Button:SetPoint("TOPLEFT", GILFrame2AttachUI, "BOTTOMLEFT", 0, 0)
	 _G[Button:GetName().."Text"]:SetText(L["OPT_4"])
	 _G[Button:GetName().."Text"]:SetFontObject("GameFontHighlight")
	Button:SetScript("OnShow",  function(self) self:SetChecked(GlobalIgnoreDB.sameserver == true) end)
	Button:SetScript("OnClick" ,function(self) GlobalIgnoreDB.sameserver = (self:GetChecked() or false) end) 

	Button = Tab2Frame:CreateFontString("GILFrame2ExpText", "OVERLAY", "GameFontHighlight")
	Button:SetPoint("TOPLEFT", GILFrame2SameServer, "BOTTOMLEFT", 6, -10)
	Button:SetText(L["OPT_5"])

	Button = CreateFrame("EditBox", "GILFrame2Exp", Tab2Frame, "InputBoxTemplate")
	Button:SetPoint("TOPLEFT", GILFrame2ExpText, "TOPRIGHT", 8, 6)
	Button:SetWidth(50)
	Button:SetHeight(20)
	Button:SetAutoFocus(false)
	Button:SetScript("OnShow",	    function(self) self:SetText(GlobalIgnoreDB.defexpire) end)
	Button:SetScript("OnEnterPressed",  function(self) if tonumber(self:GetText()) then GlobalIgnoreDB.defexpire = self:GetText() end self:ClearFocus() end)
	Button:SetScript("OnEscapePressed", function(self) self:SetText(GlobalIgnoreDB.defexpire) end)
	
	----------
	-- SHOW --
	----------	
	
	PanelTemplates_SetNumTabs(MainFrame, 2)
	PanelTemplates_SetTab(MainFrame, 1)
	
	MainFrame:Show()
end

function GUI()
	
	CreateUIFrames()
end

-- hook blizzard friends window

function friendsShow()
	if not GlobalIgnoreDB.openWithFriends then
		return
	end
	
	if MainFrame == nil then
		CreateUIFrames()
	end
	
	if MainFrame ~= nil then
		MainFrame:ClearAllPoints()
		
		if GlobalIgnoreDB.attachFriends then
			MainFrame:SetPoint("TOPLEFT", FriendsFrame, "TOPRIGHT", 12, -10)
		else
			MainFrame:SetPoint("CENTER", UIParent)
		end
		
		MainFrame:Show()
	end
end

function friendsHide()
	if not GlobalIgnoreDB.openWithFriends then
		return
	end

	if MainFrame ~= nil then
		MainFrame:Hide()
	end
end

FriendsFrame:HookScript("OnShow", friendsShow)
FriendsFrame:HookScript("OnHide", friendsHide)

-------------
-- THE END --
-------------